@@ -203,12 +203,6 @@ impl RegistryClient {
         self.timeout
     }
 
-    /// Set the index URLs to use for fetching packages.
-    #[must_use]
-    pub fn with_index_url(self, index_urls: IndexUrls) -> Self {
-        Self { index_urls, ..self }
-    }
-
     /// Fetch a package from the `PyPI` simple API.
     ///
     /// "simple" here refers to [PEP 503 – Simple Repository API](https://peps.python.org/pep-0503/)
@@ -850,9 +844,10 @@ impl MediaType {
     }
 }
 
-#[derive(Debug, Copy, Clone, Eq, PartialEq)]
+#[derive(Debug, Copy, Clone, Eq, PartialEq, Default)]
 pub enum Connectivity {
     /// Allow access to the network.
+    #[default]
     Online,
 
     /// Do not allow access to the network.
