@@ -92,7 +92,11 @@ impl<'a> BuiltWheelIndex<'a> {
     ) -> Result<Option<CachedWheel>, Error> {
         let cache_shard = self.cache.shard(
             CacheBucket::BuiltWheels,
-            WheelCache::Path(&source_dist.url).root(),
+            if source_dist.editable {
+                WheelCache::Editable(&source_dist.url).root()
+            } else {
+                WheelCache::Path(&source_dist.url).root()
+            },
         );
 
         // Read the revision from the cache.
@@ -105,7 +109,7 @@ impl<'a> BuiltWheelIndex<'a> {
         let Some(modified) =
             ArchiveTimestamp::from_source_tree(&source_dist.path).map_err(Error::CacheRead)?
         else {
-            return Err(Error::DirWithoutEntrypoint);
+            return Err(Error::DirWithoutEntrypoint(source_dist.path.clone()));
         };
 
         // If the distribution is stale, omit it from the index.
