@@ -3,7 +3,6 @@ use std::path::Path;
 use std::str::FromStr;
 use std::sync::Arc;
 
-use anyhow::Result;
 use configparser::ini::Ini;
 use futures::{stream::FuturesOrdered, TryStreamExt};
 use serde::Deserialize;
@@ -12,16 +11,28 @@ use tracing::debug;
 use distribution_filename::{SourceDistFilename, WheelFilename};
 use distribution_types::{
     BuildableSource, DirectSourceUrl, DirectorySourceUrl, GitSourceUrl, PathSourceUrl,
-    RemoteSource, Requirement, SourceUrl, UnresolvedRequirement,
-    UnresolvedRequirementSpecification, VersionId,
+    RemoteSource, SourceUrl, UnresolvedRequirement, UnresolvedRequirementSpecification, VersionId,
 };
 use pep508_rs::{UnnamedRequirement, VersionOrUrl};
+use pypi_types::Requirement;
 use pypi_types::{Metadata10, ParsedUrl, VerbatimParsedUrl};
 use uv_distribution::{DistributionDatabase, Reporter};
 use uv_normalize::PackageName;
 use uv_resolver::{InMemoryIndex, MetadataResponse};
 use uv_types::{BuildContext, HashStrategy};
 
+#[derive(Debug, thiserror::Error)]
+pub enum NamedRequirementsError {
+    #[error(transparent)]
+    Distribution(#[from] uv_distribution::Error),
+
+    #[error(transparent)]
+    DistributionTypes(#[from] distribution_types::Error),
+
+    #[error(transparent)]
+    WheelFilename(#[from] distribution_filename::WheelFilenameError),
+}
+
 /// Like [`RequirementsSpecification`], but with concrete names for all requirements.
 pub struct NamedRequirementsResolver<'a, Context: BuildContext> {
     /// The requirements for the project.
@@ -60,7 +71,7 @@ impl<'a, Context: BuildContext> NamedRequirementsResolver<'a, Context> {
     }
 
     /// Resolve any unnamed requirements in the specification.
-    pub async fn resolve(self) -> Result<Vec<Requirement>> {
+    pub async fn resolve(self) -> Result<Vec<Requirement>, NamedRequirementsError> {
         let Self {
             requirements,
             hasher,
@@ -88,7 +99,7 @@ impl<'a, Context: BuildContext> NamedRequirementsResolver<'a, Context> {
         hasher: &HashStrategy,
         index: &InMemoryIndex,
         database: &DistributionDatabase<'a, Context>,
-    ) -> Result<pep508_rs::Requirement<VerbatimParsedUrl>> {
+    ) -> Result<pep508_rs::Requirement<VerbatimParsedUrl>, NamedRequirementsError> {
         // If the requirement is a wheel, extract the package name from the wheel filename.
         //
         // Ex) `anyio-4.3.0-py3-none-any.whl`
