@@ -6,7 +6,6 @@ use std::ops::Deref;
 use std::sync::Arc;
 use std::thread;
 
-use anyhow::Result;
 use dashmap::DashMap;
 use futures::{FutureExt, StreamExt, TryFutureExt};
 use itertools::Itertools;
@@ -21,18 +20,18 @@ use tracing::{debug, enabled, instrument, trace, warn, Level};
 
 use distribution_types::{
     BuiltDist, Dist, DistributionMetadata, IncompatibleDist, IncompatibleSource, IncompatibleWheel,
-    InstalledDist, RemoteSource, Requirement, ResolvedDist, ResolvedDistRef, SourceDist,
-    VersionOrUrlRef,
+    InstalledDist, RemoteSource, ResolvedDist, ResolvedDistRef, SourceDist, VersionOrUrlRef,
 };
 pub(crate) use locals::Locals;
 use pep440_rs::{Version, MIN_VERSION};
 use pep508_rs::MarkerEnvironment;
 use platform_tags::Tags;
-use pypi_types::Metadata23;
+use pypi_types::{Metadata23, Requirement};
 pub(crate) use urls::Urls;
 use uv_configuration::{Constraints, Overrides};
 use uv_distribution::{ArchiveMetadata, DistributionDatabase};
-use uv_normalize::PackageName;
+use uv_git::GitResolver;
+use uv_normalize::{ExtraName, PackageName};
 use uv_types::{BuildContext, HashStrategy, InstalledPackagesProvider};
 
 use crate::candidate_selector::{CandidateDist, CandidateSelector};
@@ -83,6 +82,7 @@ struct ResolverState<InstalledPackages: InstalledPackagesProvider> {
     constraints: Constraints,
     overrides: Overrides,
     preferences: Preferences,
+    git: GitResolver,
     exclusions: Exclusions,
     urls: Urls,
     locals: Locals,
@@ -155,6 +155,7 @@ impl<'a, Context: BuildContext, InstalledPackages: InstalledPackagesProvider>
             markers,
             python_requirement,
             index,
+            build_context.git(),
             provider,
             installed_packages,
         )
@@ -173,16 +174,18 @@ impl<Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvider>
         markers: Option<&MarkerEnvironment>,
         python_requirement: &PythonRequirement,
         index: &InMemoryIndex,
+        git: &GitResolver,
         provider: Provider,
         installed_packages: InstalledPackages,
     ) -> Result<Self, ResolveError> {
         let state = ResolverState {
             index: index.clone(),
+            git: git.clone(),
             unavailable_packages: DashMap::default(),
             incomplete_packages: DashMap::default(),
             selector: CandidateSelector::for_resolution(options, &manifest, markers),
             dependency_mode: options.dependency_mode,
-            urls: Urls::from_manifest(&manifest, markers, options.dependency_mode)?,
+            urls: Urls::from_manifest(&manifest, markers, git, options.dependency_mode)?,
             locals: Locals::from_manifest(&manifest, markers, options.dependency_mode),
             project: manifest.project,
             requirements: manifest.requirements,
@@ -297,221 +300,248 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
     ) -> Result<ResolutionGraph, ResolveError> {
         let root = PubGrubPackage::from(PubGrubPackageInner::Root(self.project.clone()));
         let mut prefetcher = BatchPrefetcher::default();
-        let mut state = SolveState {
+        let state = SolveState {
             pubgrub: State::init(root.clone(), MIN_VERSION.clone()),
             next: root,
             pins: FilePins::default(),
             priorities: PubGrubPriorities::default(),
             added_dependencies: FxHashMap::default(),
         };
+        let mut forked_states = vec![state];
+        let mut resolutions = vec![];
 
         debug!(
             "Solving with target Python version {}",
             self.python_requirement.target()
         );
 
-        loop {
-            // Run unit propagation.
-            state.pubgrub.unit_propagation(state.next)?;
+        'FORK: while let Some(mut state) = forked_states.pop() {
+            loop {
+                // Run unit propagation.
+                state.pubgrub.unit_propagation(state.next.clone())?;
 
-            // Pre-visit all candidate packages, to allow metadata to be fetched in parallel. If
-            // the dependency mode is direct, we only need to visit the root package.
-            if self.dependency_mode.is_transitive() {
-                Self::pre_visit(
-                    state.pubgrub.partial_solution.prioritized_packages(),
+                // Pre-visit all candidate packages, to allow metadata to be fetched in parallel. If
+                // the dependency mode is direct, we only need to visit the root package.
+                if self.dependency_mode.is_transitive() {
+                    Self::pre_visit(
+                        state.pubgrub.partial_solution.prioritized_packages(),
+                        &request_sink,
+                    )?;
+                }
+
+                // Choose a package version.
+                let Some(highest_priority_pkg) = state
+                    .pubgrub
+                    .partial_solution
+                    .pick_highest_priority_pkg(|package, _range| state.priorities.get(package))
+                else {
+                    if enabled!(Level::DEBUG) {
+                        prefetcher.log_tried_versions();
+                    }
+                    resolutions.push(state.into_resolution());
+                    continue 'FORK;
+                };
+                state.next = highest_priority_pkg;
+
+                prefetcher.version_tried(state.next.clone());
+
+                let term_intersection = state
+                    .pubgrub
+                    .partial_solution
+                    .term_intersection_for_package(&state.next)
+                    .ok_or_else(|| {
+                        PubGrubError::Failure(
+                            "a package was chosen but we don't have a term.".into(),
+                        )
+                    })?;
+                let decision = self.choose_version(
+                    &state.next,
+                    term_intersection.unwrap_positive(),
+                    &mut state.pins,
+                    visited,
                     &request_sink,
                 )?;
-            }
 
-            // Choose a package version.
-            let Some(highest_priority_pkg) = state
-                .pubgrub
-                .partial_solution
-                .pick_highest_priority_pkg(|package, _range| state.priorities.get(package))
-            else {
-                if enabled!(Level::DEBUG) {
-                    prefetcher.log_tried_versions();
-                }
-                let selection = state.pubgrub.partial_solution.extract_solution();
-                return ResolutionGraph::from_state(
-                    &selection,
-                    &state.pins,
-                    self.index.packages(),
-                    self.index.distributions(),
-                    &state.pubgrub,
-                    &self.preferences,
-                );
-            };
-            state.next = highest_priority_pkg;
+                // Pick the next compatible version.
+                let version = match decision {
+                    None => {
+                        debug!("No compatible version found for: {next}", next = state.next);
 
-            prefetcher.version_tried(state.next.clone());
-
-            let term_intersection = state
-                .pubgrub
-                .partial_solution
-                .term_intersection_for_package(&state.next)
-                .ok_or_else(|| {
-                    PubGrubError::Failure("a package was chosen but we don't have a term.".into())
-                })?;
-            let decision = self.choose_version(
-                &state.next,
-                term_intersection.unwrap_positive(),
-                &mut state.pins,
-                visited,
-                &request_sink,
-            )?;
-
-            // Pick the next compatible version.
-            let version = match decision {
-                None => {
-                    debug!("No compatible version found for: {next}", next = state.next);
-
-                    let term_intersection = state
-                        .pubgrub
-                        .partial_solution
-                        .term_intersection_for_package(&state.next)
-                        .expect("a package was chosen but we don't have a term.");
-
-                    // Check if the decision was due to the package being unavailable
-                    if let PubGrubPackageInner::Package { ref name, .. } = &*state.next {
-                        if let Some(entry) = self.unavailable_packages.get(name) {
-                            state
-                                .pubgrub
-                                .add_incompatibility(Incompatibility::custom_term(
-                                    state.next.clone(),
-                                    term_intersection.clone(),
-                                    UnavailableReason::Package(entry.clone()),
-                                ));
-                            continue;
-                        }
-                    }
-
-                    state
-                        .pubgrub
-                        .add_incompatibility(Incompatibility::no_versions(
-                            state.next.clone(),
-                            term_intersection.clone(),
-                        ));
-                    continue;
-                }
-                Some(version) => version,
-            };
-            let version = match version {
-                ResolverVersion::Available(version) => version,
-                ResolverVersion::Unavailable(version, reason) => {
-                    // Incompatible requires-python versions are special in that we track
-                    // them as incompatible dependencies instead of marking the package version
-                    // as unavailable directly
-                    if let UnavailableVersion::IncompatibleDist(
-                        IncompatibleDist::Source(IncompatibleSource::RequiresPython(
-                            requires_python,
-                        ))
-                        | IncompatibleDist::Wheel(IncompatibleWheel::RequiresPython(requires_python)),
-                    ) = reason
-                    {
-                        let python_version = requires_python
-                            .iter()
-                            .map(PubGrubSpecifier::try_from)
-                            .fold_ok(Range::full(), |range, specifier| {
-                                range.intersection(&specifier.into())
-                            })?;
-
-                        let package = &state.next;
-                        for kind in [PubGrubPython::Installed, PubGrubPython::Target] {
-                            state
-                                .pubgrub
-                                .add_incompatibility(Incompatibility::from_dependency(
-                                    package.clone(),
-                                    Range::singleton(version.clone()),
-                                    (
-                                        PubGrubPackage::from(PubGrubPackageInner::Python(kind)),
-                                        python_version.clone(),
-                                    ),
-                                ));
-                        }
-                        state
+                        let term_intersection = state
                             .pubgrub
                             .partial_solution
-                            .add_decision(state.next.clone(), version);
+                            .term_intersection_for_package(&state.next)
+                            .expect("a package was chosen but we don't have a term.");
+
+                        // Check if the decision was due to the package being unavailable
+                        if let PubGrubPackageInner::Package { ref name, .. } = &*state.next {
+                            if let Some(entry) = self.unavailable_packages.get(name) {
+                                state
+                                    .pubgrub
+                                    .add_incompatibility(Incompatibility::custom_term(
+                                        state.next.clone(),
+                                        term_intersection.clone(),
+                                        UnavailableReason::Package(entry.clone()),
+                                    ));
+                                continue;
+                            }
+                        }
+
+                        state
+                            .pubgrub
+                            .add_incompatibility(Incompatibility::no_versions(
+                                state.next.clone(),
+                                term_intersection.clone(),
+                            ));
                         continue;
-                    };
-                    state
-                        .pubgrub
-                        .add_incompatibility(Incompatibility::custom_version(
-                            state.next.clone(),
-                            version.clone(),
-                            UnavailableReason::Version(reason),
-                        ));
-                    continue;
-                }
-            };
+                    }
+                    Some(version) => version,
+                };
+                let version = match version {
+                    ResolverVersion::Available(version) => version,
+                    ResolverVersion::Unavailable(version, reason) => {
+                        // Incompatible requires-python versions are special in that we track
+                        // them as incompatible dependencies instead of marking the package version
+                        // as unavailable directly
+                        if let UnavailableVersion::IncompatibleDist(
+                            IncompatibleDist::Source(IncompatibleSource::RequiresPython(
+                                requires_python,
+                            ))
+                            | IncompatibleDist::Wheel(IncompatibleWheel::RequiresPython(
+                                requires_python,
+                            )),
+                        ) = reason
+                        {
+                            let python_version = requires_python
+                                .iter()
+                                .map(PubGrubSpecifier::try_from)
+                                .fold_ok(Range::full(), |range, specifier| {
+                                    range.intersection(&specifier.into())
+                                })?;
 
-            prefetcher.prefetch_batches(
-                &state.next,
-                &version,
-                term_intersection.unwrap_positive(),
-                &request_sink,
-                &self.index,
-                &self.selector,
-            )?;
-
-            self.on_progress(&state.next, &version);
-
-            if state
-                .added_dependencies
-                .entry(state.next.clone())
-                .or_default()
-                .insert(version.clone())
-            {
-                // Retrieve that package dependencies.
-                let package = &state.next;
-                let dependencies = match self.get_dependencies(
-                    package,
-                    &version,
-                    &mut state.priorities,
-                    &request_sink,
-                )? {
-                    Dependencies::Unavailable(reason) => {
+                            let package = &state.next;
+                            for kind in [PubGrubPython::Installed, PubGrubPython::Target] {
+                                state.pubgrub.add_incompatibility(
+                                    Incompatibility::from_dependency(
+                                        package.clone(),
+                                        Range::singleton(version.clone()),
+                                        (
+                                            PubGrubPackage::from(PubGrubPackageInner::Python(kind)),
+                                            python_version.clone(),
+                                        ),
+                                    ),
+                                );
+                            }
+                            state
+                                .pubgrub
+                                .partial_solution
+                                .add_decision(state.next.clone(), version);
+                            continue;
+                        };
                         state
                             .pubgrub
                             .add_incompatibility(Incompatibility::custom_version(
-                                package.clone(),
+                                state.next.clone(),
                                 version.clone(),
                                 UnavailableReason::Version(reason),
                             ));
                         continue;
                     }
-                    Dependencies::Available(constraints)
-                        if constraints
-                            .iter()
-                            .any(|(dependency, _)| dependency == package) =>
-                    {
-                        if enabled!(Level::DEBUG) {
-                            prefetcher.log_tried_versions();
-                        }
-                        return Err(PubGrubError::SelfDependency {
-                            package: package.clone(),
-                            version: version.clone(),
-                        }
-                        .into());
-                    }
-                    Dependencies::Available(constraints) => constraints,
                 };
 
-                // Add that package and version if the dependencies are not problematic.
-                let dep_incompats = state.pubgrub.add_incompatibility_from_dependencies(
-                    package.clone(),
-                    version.clone(),
-                    dependencies,
-                );
+                prefetcher.prefetch_batches(
+                    &state.next,
+                    &version,
+                    term_intersection.unwrap_positive(),
+                    &request_sink,
+                    &self.index,
+                    &self.selector,
+                )?;
 
-                state.pubgrub.partial_solution.add_version(
-                    package.clone(),
-                    version,
-                    dep_incompats,
-                    &state.pubgrub.incompatibility_store,
-                );
-            } else {
+                self.on_progress(&state.next, &version);
+
+                if state
+                    .added_dependencies
+                    .entry(state.next.clone())
+                    .or_default()
+                    .insert(version.clone())
+                {
+                    // Retrieve that package dependencies.
+                    let package = state.next.clone();
+                    let forks = self.get_dependencies_forking(
+                        &package,
+                        &version,
+                        &mut state.priorities,
+                        &request_sink,
+                    )?;
+                    let forks_len = forks.len();
+                    // This is a somewhat tortured technique to ensure
+                    // that our resolver state is only cloned as much
+                    // as it needs to be. And *especially*, in the case
+                    // when no forks occur, the state should not be
+                    // cloned at all. We basically move the state into
+                    // `forked_states`, and then only clone it if there
+                    // it at least one more fork to visit.
+                    let mut cur_state = Some(state);
+                    for (i, fork) in forks.into_iter().enumerate() {
+                        let is_last = i == forks_len - 1;
+
+                        let dependencies = match fork {
+                            Dependencies::Unavailable(reason) => {
+                                let mut forked_state = cur_state.take().unwrap();
+                                if !is_last {
+                                    cur_state = Some(forked_state.clone());
+                                }
+
+                                forked_state.pubgrub.add_incompatibility(
+                                    Incompatibility::custom_version(
+                                        package.clone(),
+                                        version.clone(),
+                                        UnavailableReason::Version(reason),
+                                    ),
+                                );
+                                forked_states.push(forked_state);
+                                continue;
+                            }
+                            Dependencies::Available(constraints)
+                                if constraints
+                                    .iter()
+                                    .any(|(dependency, _)| dependency == &package) =>
+                            {
+                                if enabled!(Level::DEBUG) {
+                                    prefetcher.log_tried_versions();
+                                }
+                                return Err(PubGrubError::SelfDependency {
+                                    package: package.clone(),
+                                    version: version.clone(),
+                                }
+                                .into());
+                            }
+                            Dependencies::Available(constraints) => constraints,
+                        };
+
+                        let mut forked_state = cur_state.take().unwrap();
+                        if !is_last {
+                            cur_state = Some(forked_state.clone());
+                        }
+
+                        // Add that package and version if the dependencies are not problematic.
+                        let dep_incompats =
+                            forked_state.pubgrub.add_incompatibility_from_dependencies(
+                                package.clone(),
+                                version.clone(),
+                                dependencies,
+                            );
+                        forked_state.pubgrub.partial_solution.add_version(
+                            package.clone(),
+                            version.clone(),
+                            dep_incompats,
+                            &forked_state.pubgrub.incompatibility_store,
+                        );
+                        forked_states.push(forked_state);
+                    }
+                    continue 'FORK;
+                }
                 // `dep_incompats` are already in `incompatibilities` so we know there are not satisfied
                 // terms and can add the decision directly.
                 state
@@ -520,6 +550,11 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     .add_decision(state.next.clone(), version);
             }
         }
+        let mut combined = Resolution::default();
+        for resolution in resolutions {
+            combined.union(resolution);
+        }
+        ResolutionGraph::from_state(&self.index, &self.preferences, &self.git, combined)
     }
 
     /// Visit a [`PubGrubPackage`] prior to selection. This should be called on a [`PubGrubPackage`]
@@ -578,7 +613,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
             let PubGrubPackageInner::Package {
                 name,
                 extra: None,
-                marker: None,
+                marker: _marker,
                 url: None,
             } = &**package
             else {
@@ -796,6 +831,67 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
         }
     }
 
+    /// Given a candidate package and version, return its dependencies.
+    #[instrument(skip_all, fields(%package, %version))]
+    fn get_dependencies_forking(
+        &self,
+        package: &PubGrubPackage,
+        version: &Version,
+        priorities: &mut PubGrubPriorities,
+        request_sink: &Sender<Request>,
+    ) -> Result<Vec<Dependencies>, ResolveError> {
+        type Dep = (PubGrubPackage, Range<Version>);
+
+        let result = self.get_dependencies(package, version, priorities, request_sink);
+        if self.markers.is_some() {
+            return result.map(|deps| vec![deps]);
+        }
+        let deps: Vec<Dep> = match result? {
+            Dependencies::Available(deps) => deps,
+            Dependencies::Unavailable(err) => return Ok(vec![Dependencies::Unavailable(err)]),
+        };
+
+        let mut by_grouping: FxHashMap<&PackageName, FxHashMap<&Range<Version>, Vec<&Dep>>> =
+            FxHashMap::default();
+        for dep in &deps {
+            let (ref pkg, ref range) = *dep;
+            let name = match &**pkg {
+                // A root can never be a dependency of another package, and a `Python` pubgrub
+                // package is never returned by `get_dependencies`. So these cases never occur.
+                PubGrubPackageInner::Root(_) | PubGrubPackageInner::Python(_) => unreachable!(),
+                PubGrubPackageInner::Package { ref name, .. }
+                | PubGrubPackageInner::Extra { ref name, .. } => name,
+            };
+            by_grouping
+                .entry(name)
+                .or_default()
+                .entry(range)
+                .or_default()
+                .push(dep);
+        }
+        let mut forks: Vec<Vec<Dep>> = vec![vec![]];
+        for (_, groups) in by_grouping {
+            if groups.len() <= 1 {
+                for deps in groups.into_values() {
+                    for fork in &mut forks {
+                        fork.extend(deps.iter().map(|dep| (*dep).clone()));
+                    }
+                }
+            } else {
+                let mut new_forks: Vec<Vec<Dep>> = vec![];
+                for deps in groups.into_values() {
+                    let mut new_forks_for_group = forks.clone();
+                    for fork in &mut new_forks_for_group {
+                        fork.extend(deps.iter().map(|dep| (*dep).clone()));
+                    }
+                    new_forks.extend(new_forks_for_group);
+                }
+                forks = new_forks;
+            }
+        }
+        Ok(forks.into_iter().map(Dependencies::Available).collect())
+    }
+
     /// Given a candidate package and version, return its dependencies.
     #[instrument(skip_all, fields(%package, %version))]
     fn get_dependencies(
@@ -816,6 +912,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     None,
                     &self.urls,
                     &self.locals,
+                    &self.git,
                     self.markers.as_ref(),
                 );
 
@@ -846,14 +943,13 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
             PubGrubPackageInner::Package {
                 name,
                 extra,
-                marker: _marker,
+                marker,
                 url,
             } => {
                 // If we're excluding transitive dependencies, short-circuit.
                 if self.dependency_mode.is_direct() {
                     // If an extra is provided, wait for the metadata to be available, since it's
                     // still required for generating the lock file.
-
                     let dist = match url {
                         Some(url) => PubGrubDistribution::from_url(name, url),
                         None => PubGrubDistribution::from_registry(name, version),
@@ -952,7 +1048,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     .cloned()
                     .map(Requirement::from)
                     .collect();
-                let dependencies = PubGrubDependencies::from_requirements(
+                let mut dependencies = PubGrubDependencies::from_requirements(
                     &requirements,
                     &self.constraints,
                     &self.overrides,
@@ -960,6 +1056,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     extra.as_ref(),
                     &self.urls,
                     &self.locals,
+                    &self.git,
                     self.markers.as_ref(),
                 )?;
 
@@ -973,6 +1070,29 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     self.visit_package(dep_package, request_sink)?;
                 }
 
+                // If a package has a marker, add a dependency from it to the
+                // same package without markers.
+                //
+                // At time of writing, AG doesn't fully understand why we need
+                // this, but one explanation is that without it, there is no
+                // way to connect two different `PubGrubPackage` values with
+                // the same package name but different markers. With different
+                // markers, they would be considered wholly distinct packages.
+                // But this dependency-on-itself-without-markers forces PubGrub
+                // to unify the constraints across what would otherwise be two
+                // distinct packages.
+                if marker.is_some() {
+                    dependencies.push(
+                        PubGrubPackage::from(PubGrubPackageInner::Package {
+                            name: name.clone(),
+                            extra: extra.clone(),
+                            marker: None,
+                            url: url.clone(),
+                        }),
+                        Range::singleton(version.clone()),
+                    );
+                }
+
                 Ok(Dependencies::Available(dependencies.into()))
             }
 
@@ -980,14 +1100,14 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
             PubGrubPackageInner::Extra {
                 name,
                 extra,
-                marker: _marker,
+                marker,
                 url,
             } => Ok(Dependencies::Available(vec![
                 (
                     PubGrubPackage::from(PubGrubPackageInner::Package {
                         name: name.clone(),
                         extra: None,
-                        marker: None,
+                        marker: marker.clone(),
                         url: url.clone(),
                     }),
                     Range::singleton(version.clone()),
@@ -996,7 +1116,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     PubGrubPackage::from(PubGrubPackageInner::Package {
                         name: name.clone(),
                         extra: Some(extra.clone()),
-                        marker: None,
+                        marker: marker.clone(),
                         url: url.clone(),
                     }),
                     Range::singleton(version.clone()),
@@ -1030,7 +1150,9 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     trace!("Received installed distribution metadata for: {dist}");
                     self.index.distributions().done(
                         dist.version_id(),
-                        Arc::new(MetadataResponse::Found(ArchiveMetadata::from(metadata))),
+                        Arc::new(MetadataResponse::Found(ArchiveMetadata::from_metadata23(
+                            metadata,
+                        ))),
                     );
                 }
                 Some(Response::Dist {
@@ -1286,6 +1408,142 @@ struct SolveState {
     added_dependencies: FxHashMap<PubGrubPackage, FxHashSet<Version>>,
 }
 
+impl SolveState {
+    fn into_resolution(self) -> Resolution {
+        let packages = self.pubgrub.partial_solution.extract_solution();
+        let mut dependencies: FxHashMap<
+            ResolutionDependencyNames,
+            FxHashSet<ResolutionDependencyVersions>,
+        > = FxHashMap::default();
+        for (package, self_version) in &packages {
+            for id in &self.pubgrub.incompatibilities[package] {
+                let pubgrub::solver::Kind::FromDependencyOf(
+                    ref self_package,
+                    ref self_range,
+                    ref dependency_package,
+                    ref dependency_range,
+                ) = self.pubgrub.incompatibility_store[*id].kind
+                else {
+                    continue;
+                };
+                if package != self_package {
+                    continue;
+                }
+                if !self_range.contains(self_version) {
+                    continue;
+                }
+                let Some(dependency_version) = packages.get(dependency_package) else {
+                    continue;
+                };
+                if !dependency_range.contains(dependency_version) {
+                    continue;
+                }
+
+                let PubGrubPackageInner::Package {
+                    name: ref self_name,
+                    extra: ref self_extra,
+                    ..
+                } = &**self_package
+                else {
+                    continue;
+                };
+
+                match **dependency_package {
+                    PubGrubPackageInner::Package {
+                        name: ref dependency_name,
+                        extra: ref dependency_extra,
+                        ..
+                    } => {
+                        if self_name == dependency_name {
+                            continue;
+                        }
+                        let names = ResolutionDependencyNames {
+                            from: self_name.clone(),
+                            to: dependency_name.clone(),
+                        };
+                        let versions = ResolutionDependencyVersions {
+                            from_version: self_version.clone(),
+                            from_extra: self_extra.clone(),
+                            to_version: dependency_version.clone(),
+                            to_extra: dependency_extra.clone(),
+                        };
+                        dependencies.entry(names).or_default().insert(versions);
+                    }
+
+                    PubGrubPackageInner::Extra {
+                        name: ref dependency_name,
+                        extra: ref dependency_extra,
+                        ..
+                    } => {
+                        if self_name == dependency_name {
+                            continue;
+                        }
+                        let names = ResolutionDependencyNames {
+                            from: self_name.clone(),
+                            to: dependency_name.clone(),
+                        };
+                        let versions = ResolutionDependencyVersions {
+                            from_version: self_version.clone(),
+                            from_extra: self_extra.clone(),
+                            to_version: dependency_version.clone(),
+                            to_extra: Some(dependency_extra.clone()),
+                        };
+                        dependencies.entry(names).or_default().insert(versions);
+                    }
+
+                    _ => {}
+                }
+            }
+        }
+        let packages = packages
+            .into_iter()
+            .map(|(package, version)| (package, FxHashSet::from_iter([version])))
+            .collect();
+        Resolution {
+            packages,
+            dependencies,
+            pins: self.pins,
+        }
+    }
+}
+
+#[derive(Debug, Default)]
+pub(crate) struct Resolution {
+    pub(crate) packages: FxHashMap<PubGrubPackage, FxHashSet<Version>>,
+    pub(crate) dependencies:
+        FxHashMap<ResolutionDependencyNames, FxHashSet<ResolutionDependencyVersions>>,
+    pub(crate) pins: FilePins,
+}
+
+#[derive(Clone, Debug, Eq, Hash, PartialEq)]
+pub(crate) struct ResolutionDependencyNames {
+    pub(crate) from: PackageName,
+    pub(crate) to: PackageName,
+}
+
+#[derive(Clone, Debug, Eq, Hash, PartialEq)]
+pub(crate) struct ResolutionDependencyVersions {
+    pub(crate) from_version: Version,
+    pub(crate) from_extra: Option<ExtraName>,
+    pub(crate) to_version: Version,
+    pub(crate) to_extra: Option<ExtraName>,
+}
+
+impl Resolution {
+    fn union(&mut self, other: Resolution) {
+        for (other_package, other_versions) in other.packages {
+            self.packages
+                .entry(other_package)
+                .or_default()
+                .extend(other_versions);
+        }
+        for (names, versions) in other.dependencies {
+            self.dependencies.entry(names).or_default().extend(versions);
+        }
+        self.pins.union(other.pins);
+    }
+}
+
 /// Fetch the metadata for an item
 #[derive(Debug)]
 #[allow(clippy::large_enum_variant)]
