@@ -8,25 +8,29 @@ use std::str::FromStr;
 
 use anstream::{eprint, AutoStream, StripStream};
 use anyhow::{anyhow, Result};
-use fs_err as fs;
 use itertools::Itertools;
 use owo_colors::OwoColorize;
+use pypi_types::Requirement;
 use tracing::debug;
 
-use distribution_types::{IndexLocations, SourceAnnotation, SourceAnnotations, Verbatim};
+use distribution_types::{
+    IndexLocations, SourceAnnotation, SourceAnnotations, UnresolvedRequirementSpecification,
+    Verbatim,
+};
 use install_wheel_rs::linker::LinkMode;
 use platform_tags::Tags;
 use uv_auth::store_credentials_from_url;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{
-    Concurrency, ConfigSettings, Constraints, IndexStrategy, NoBinary, NoBuild, Overrides,
-    PreviewMode, SetupPyStrategy, Upgrade,
+    Concurrency, ConfigSettings, Constraints, ExtrasSpecification, IndexStrategy, NoBinary,
+    NoBuild, Overrides, PreviewMode, SetupPyStrategy, Upgrade,
 };
 use uv_configuration::{KeyringProviderType, TargetTriple};
 use uv_dispatch::BuildDispatch;
 use uv_distribution::DistributionDatabase;
 use uv_fs::Simplified;
+use uv_git::GitResolver;
 use uv_interpreter::{
     find_best_interpreter, find_interpreter, InterpreterRequest, PythonEnvironment, SystemPython,
     VersionRequest,
@@ -34,7 +38,7 @@ use uv_interpreter::{
 use uv_interpreter::{PythonVersion, SourceSelector};
 use uv_normalize::{ExtraName, PackageName};
 use uv_requirements::{
-    upgrade::read_lockfile, ExtrasSpecification, LookaheadResolver, NamedRequirementsResolver,
+    upgrade::read_requirements_txt, LookaheadResolver, NamedRequirementsResolver,
     RequirementsSource, RequirementsSpecification, SourceTreeResolver,
 };
 use uv_resolver::{
@@ -56,6 +60,7 @@ pub(crate) async fn pip_compile(
     requirements: &[RequirementsSource],
     constraints: &[RequirementsSource],
     overrides: &[RequirementsSource],
+    overrides_from_workspace: Vec<Requirement>,
     extras: ExtrasSpecification,
     output_file: Option<&Path>,
     resolution_mode: ResolutionMode,
@@ -88,7 +93,6 @@ pub(crate) async fn pip_compile(
     python: Option<String>,
     system: bool,
     concurrency: Concurrency,
-    uv_lock: bool,
     native_tls: bool,
     quiet: bool,
     preview: PreviewMode,
@@ -112,7 +116,7 @@ pub(crate) async fn pip_compile(
 
     // Read all requirements from the provided sources.
     let RequirementsSpecification {
-        project,
+        mut project,
         requirements,
         constraints,
         overrides,
@@ -128,10 +132,7 @@ pub(crate) async fn pip_compile(
         requirements,
         constraints,
         overrides,
-        None,
-        &extras,
         &client_builder,
-        preview,
     )
     .await?;
 
@@ -249,15 +250,7 @@ pub(crate) async fn pip_compile(
         (None, Some(python_version)) => Cow::Owned(python_version.markers(interpreter.markers())),
         (None, None) => Cow::Borrowed(interpreter.markers()),
     };
-    // The marker environment to use for evaluating requirements. When
-    // `uv_lock` is enabled, we specifically do environment independent marker
-    // evaluation. (i.e., Only consider extras.)
-    let marker_filter = if uv_lock { None } else { Some(&*markers) };
-    // The Python requirement in "workspace-aware uv" should, I believe, come
-    // from the pyproject.toml. For now, we just take it from the markers
-    // (which does have its Python version set potentially from the CLI, which
-    // I think is spiritually equivalent to setting the Python version in
-    // pyproject.toml).
+
     let python_requirement = PythonRequirement::from_marker_environment(&interpreter, &markers);
 
     // Generate, but don't enforce hashes for the requirements.
@@ -288,7 +281,8 @@ pub(crate) async fn pip_compile(
         .build();
 
     // Read the lockfile, if present.
-    let preferences = read_lockfile(output_file, upgrade).await?;
+    let preferences = read_requirements_txt(output_file, &upgrade).await?;
+    let git = GitResolver::default();
 
     // Resolve the flat indexes from `--find-links`.
     let flat_index = {
@@ -319,6 +313,7 @@ pub(crate) async fn pip_compile(
         &index_locations,
         &flat_index,
         &source_index,
+        &git,
         &in_flight,
         setup_py,
         &config_settings,
@@ -327,6 +322,7 @@ pub(crate) async fn pip_compile(
         &no_build,
         &NoBinary::None,
         concurrency,
+        preview,
     )
     .with_options(OptionsBuilder::new().exclude_newer(exclude_newer).build());
 
@@ -337,7 +333,7 @@ pub(crate) async fn pip_compile(
             requirements,
             &hasher,
             &top_level_index,
-            DistributionDatabase::new(&client, &build_dispatch, concurrency.downloads),
+            DistributionDatabase::new(&client, &build_dispatch, concurrency.downloads, preview),
         )
         .with_reporter(ResolverReporter::from(printer))
         .resolve()
@@ -345,29 +341,75 @@ pub(crate) async fn pip_compile(
 
         // Resolve any source trees into requirements.
         if !source_trees.is_empty() {
+            let resolutions = SourceTreeResolver::new(
+                source_trees,
+                &extras,
+                &hasher,
+                &top_level_index,
+                DistributionDatabase::new(&client, &build_dispatch, concurrency.downloads, preview),
+            )
+            .with_reporter(ResolverReporter::from(printer))
+            .resolve()
+            .await?;
+
+            // If we resolved a single project, use it for the project name.
+            project = project.or_else(|| {
+                if let [resolution] = &resolutions[..] {
+                    Some(resolution.project.clone())
+                } else {
+                    None
+                }
+            });
+
+            // If any of the extras were unused, surface a warning.
+            if let ExtrasSpecification::Some(extras) = extras {
+                let mut unused_extras = extras
+                    .iter()
+                    .filter(|extra| {
+                        !resolutions
+                            .iter()
+                            .any(|resolution| resolution.extras.contains(extra))
+                    })
+                    .collect::<Vec<_>>();
+                if !unused_extras.is_empty() {
+                    unused_extras.sort_unstable();
+                    unused_extras.dedup();
+                    let s = if unused_extras.len() == 1 { "" } else { "s" };
+                    return Err(anyhow!(
+                        "Requested extra{s} not found: {}",
+                        unused_extras.iter().join(", ")
+                    ));
+                }
+            }
+
+            // Extend the requirements with the resolved source trees.
             requirements.extend(
-                SourceTreeResolver::new(
-                    source_trees,
-                    &extras,
-                    &hasher,
-                    &top_level_index,
-                    DistributionDatabase::new(&client, &build_dispatch, concurrency.downloads),
-                )
-                .with_reporter(ResolverReporter::from(printer))
-                .resolve()
-                .await?,
+                resolutions
+                    .into_iter()
+                    .flat_map(|resolution| resolution.requirements),
             );
         }
 
         requirements
     };
 
+    // Merge workspace overrides.
+    let overrides: Vec<UnresolvedRequirementSpecification> = overrides
+        .iter()
+        .cloned()
+        .chain(
+            overrides_from_workspace
+                .into_iter()
+                .map(UnresolvedRequirementSpecification::from),
+        )
+        .collect();
+
     // Resolve the overrides from the provided sources.
     let overrides = NamedRequirementsResolver::new(
         overrides,
         &hasher,
         &top_level_index,
-        DistributionDatabase::new(&client, &build_dispatch, concurrency.downloads),
+        DistributionDatabase::new(&client, &build_dispatch, concurrency.downloads, preview),
     )
     .with_reporter(ResolverReporter::from(printer))
     .resolve()
@@ -378,7 +420,7 @@ pub(crate) async fn pip_compile(
 
     for requirement in requirements
         .iter()
-        .filter(|requirement| requirement.evaluate_markers(marker_filter, &[]))
+        .filter(|requirement| requirement.evaluate_markers(Some(&markers), &[]))
     {
         if let Some(origin) = &requirement.origin {
             sources.add(
@@ -390,7 +432,7 @@ pub(crate) async fn pip_compile(
 
     for requirement in constraints
         .iter()
-        .filter(|requirement| requirement.evaluate_markers(marker_filter, &[]))
+        .filter(|requirement| requirement.evaluate_markers(Some(&markers), &[]))
     {
         if let Some(origin) = &requirement.origin {
             sources.add(
@@ -402,7 +444,7 @@ pub(crate) async fn pip_compile(
 
     for requirement in overrides
         .iter()
-        .filter(|requirement| requirement.evaluate_markers(marker_filter, &[]))
+        .filter(|requirement| requirement.evaluate_markers(Some(&markers), &[]))
     {
         if let Some(origin) = &requirement.origin {
             sources.add(
@@ -425,10 +467,10 @@ pub(crate) async fn pip_compile(
                 &overrides,
                 &hasher,
                 &top_level_index,
-                DistributionDatabase::new(&client, &build_dispatch, concurrency.downloads),
+                DistributionDatabase::new(&client, &build_dispatch, concurrency.downloads, preview),
             )
             .with_reporter(ResolverReporter::from(printer))
-            .resolve(marker_filter)
+            .resolve(Some(&markers))
             .await?
         }
         DependencyMode::Direct => Vec::new(),
@@ -459,14 +501,14 @@ pub(crate) async fn pip_compile(
         manifest.clone(),
         options,
         &python_requirement,
-        marker_filter,
+        Some(&markers),
         &tags,
         &flat_index,
         &top_level_index,
         &hasher,
         &build_dispatch,
         EmptyInstalledPackages,
-        DistributionDatabase::new(&client, &build_dispatch, concurrency.downloads),
+        DistributionDatabase::new(&client, &build_dispatch, concurrency.downloads, preview),
     )?
     .with_reporter(ResolverReporter::from(printer));
 
@@ -526,12 +568,6 @@ pub(crate) async fn pip_compile(
         writeln!(writer, "{}", format!("#    {relevant_markers}").green())?;
     }
 
-    if uv_lock {
-        let lock = resolution.lock()?;
-        let encoded = toml::to_string_pretty(&lock)?;
-        fs::tokio::write("uv.lock", encoded.as_bytes()).await?;
-    }
-
     // Write the index locations to the output channel.
     let mut wrote_index = false;
 
