@@ -2,30 +2,30 @@ use std::borrow::Cow;
 use std::fmt::Write;
 
 use anstream::eprint;
-use distribution_types::{IndexLocations, Resolution};
-use fs_err as fs;
 use itertools::Itertools;
 use owo_colors::OwoColorize;
 use tracing::{debug, enabled, Level};
 
+use distribution_types::{IndexLocations, Resolution, UnresolvedRequirementSpecification};
 use install_wheel_rs::linker::LinkMode;
 use platform_tags::Tags;
+use pypi_types::Requirement;
 use uv_auth::store_credentials_from_url;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{
-    Concurrency, ConfigSettings, IndexStrategy, NoBinary, NoBuild, PreviewMode, Reinstall,
-    SetupPyStrategy, Upgrade,
+    Concurrency, ConfigSettings, ExtrasSpecification, IndexStrategy, NoBinary, NoBuild,
+    PreviewMode, Reinstall, SetupPyStrategy, Upgrade,
 };
 use uv_configuration::{KeyringProviderType, TargetTriple};
 use uv_dispatch::BuildDispatch;
 use uv_fs::Simplified;
+use uv_git::GitResolver;
 use uv_installer::{SatisfiesResult, SitePackages};
 use uv_interpreter::{PythonEnvironment, PythonVersion, SystemPython, Target};
-use uv_normalize::PackageName;
-use uv_requirements::{ExtrasSpecification, RequirementsSource, RequirementsSpecification};
+use uv_requirements::{RequirementsSource, RequirementsSpecification};
 use uv_resolver::{
-    DependencyMode, ExcludeNewer, FlatIndex, InMemoryIndex, Lock, OptionsBuilder, PreReleaseMode,
+    DependencyMode, ExcludeNewer, FlatIndex, InMemoryIndex, OptionsBuilder, PreReleaseMode,
     ResolutionMode,
 };
 use uv_types::{BuildIsolation, HashStrategy, InFlight};
@@ -41,6 +41,7 @@ pub(crate) async fn pip_install(
     requirements: &[RequirementsSource],
     constraints: &[RequirementsSource],
     overrides: &[RequirementsSource],
+    overrides_from_workspace: Vec<Requirement>,
     extras: &ExtrasSpecification,
     resolution_mode: ResolutionMode,
     prerelease_mode: PreReleaseMode,
@@ -68,7 +69,6 @@ pub(crate) async fn pip_install(
     break_system_packages: bool,
     target: Option<Target>,
     concurrency: Concurrency,
-    uv_lock: Option<String>,
     native_tls: bool,
     preview: PreviewMode,
     cache: Cache,
@@ -100,13 +100,21 @@ pub(crate) async fn pip_install(
         requirements,
         constraints,
         overrides,
-        None,
         extras,
         &client_builder,
-        preview,
     )
     .await?;
 
+    let overrides: Vec<UnresolvedRequirementSpecification> = overrides
+        .iter()
+        .cloned()
+        .chain(
+            overrides_from_workspace
+                .into_iter()
+                .map(UnresolvedRequirementSpecification::from),
+        )
+        .collect();
+
     // Detect the current Python interpreter.
     let system = if system {
         SystemPython::Required
@@ -161,12 +169,7 @@ pub(crate) async fn pip_install(
     // Check if the current environment satisfies the requirements.
     // Ideally, the resolver would be fast enough to let us remove this check. But right now, for large environments,
     // it's an order of magnitude faster to validate the environment than to resolve the requirements.
-    if reinstall.is_none()
-        && upgrade.is_none()
-        && source_trees.is_empty()
-        && overrides.is_empty()
-        && uv_lock.is_none()
-    {
+    if reinstall.is_none() && upgrade.is_none() && source_trees.is_empty() && overrides.is_empty() {
         match site_packages.satisfies(&requirements, &constraints)? {
             // If the requirements are already satisfied, we're done.
             SatisfiesResult::Fresh {
@@ -255,6 +258,10 @@ pub(crate) async fn pip_install(
         HashStrategy::None
     };
 
+    // When resolving, don't take any external preferences into account.
+    let preferences = Vec::default();
+    let git = GitResolver::default();
+
     // Incorporate any index locations from the provided sources.
     let index_locations =
         index_locations.combine(index_url, extra_index_urls, find_links, no_index);
@@ -307,6 +314,7 @@ pub(crate) async fn pip_install(
         &index_locations,
         &flat_index,
         &index,
+        &git,
         &in_flight,
         setup_py,
         config_settings,
@@ -315,57 +323,53 @@ pub(crate) async fn pip_install(
         &no_build,
         &no_binary,
         concurrency,
+        preview,
     )
     .with_options(OptionsBuilder::new().exclude_newer(exclude_newer).build());
 
-    // Resolve the requirements.
-    let resolution = if let Some(ref root) = uv_lock {
-        let root = PackageName::new(root.to_string())?;
-        let encoded = fs::tokio::read_to_string("uv.lock").await?;
-        let lock: Lock = toml::from_str(&encoded)?;
-        lock.to_resolution(&markers, &tags, &root)
-    } else {
-        let options = OptionsBuilder::new()
-            .resolution_mode(resolution_mode)
-            .prerelease_mode(prerelease_mode)
-            .dependency_mode(dependency_mode)
-            .exclude_newer(exclude_newer)
-            .index_strategy(index_strategy)
-            .build();
+    let options = OptionsBuilder::new()
+        .resolution_mode(resolution_mode)
+        .prerelease_mode(prerelease_mode)
+        .dependency_mode(dependency_mode)
+        .exclude_newer(exclude_newer)
+        .index_strategy(index_strategy)
+        .build();
 
-        match operations::resolve(
-            requirements,
-            constraints,
-            overrides,
-            source_trees,
-            project,
-            extras,
-            site_packages.clone(),
-            &hasher,
-            &reinstall,
-            &upgrade,
-            &interpreter,
-            &tags,
-            &markers,
-            &client,
-            &flat_index,
-            &index,
-            &resolve_dispatch,
-            concurrency,
-            options,
-            printer,
-        )
-        .await
-        {
-            Ok(resolution) => Resolution::from(resolution),
-            Err(operations::Error::Resolve(uv_resolver::ResolveError::NoSolution(err))) => {
-                let report = miette::Report::msg(format!("{err}"))
-                    .context("No solution found when resolving dependencies:");
-                eprint!("{report:?}");
-                return Ok(ExitStatus::Failure);
-            }
-            Err(err) => return Err(err.into()),
+    // Resolve the requirements.
+    let resolution = match operations::resolve(
+        requirements,
+        constraints,
+        overrides,
+        source_trees,
+        project,
+        extras,
+        preferences,
+        site_packages.clone(),
+        &hasher,
+        &reinstall,
+        &upgrade,
+        &interpreter,
+        &tags,
+        Some(&markers),
+        &client,
+        &flat_index,
+        &index,
+        &resolve_dispatch,
+        concurrency,
+        options,
+        printer,
+        preview,
+    )
+    .await
+    {
+        Ok(resolution) => Resolution::from(resolution),
+        Err(operations::Error::Resolve(uv_resolver::ResolveError::NoSolution(err))) => {
+            let report = miette::Report::msg(format!("{err}"))
+                .context("No solution found when resolving dependencies:");
+            eprint!("{report:?}");
+            return Ok(ExitStatus::Failure);
         }
+        Err(err) => return Err(err.into()),
     };
 
     // Re-initialize the in-flight map.
@@ -383,6 +387,7 @@ pub(crate) async fn pip_install(
             &index_locations,
             &flat_index,
             &index,
+            &git,
             &in_flight,
             setup_py,
             config_settings,
@@ -391,6 +396,7 @@ pub(crate) async fn pip_install(
             &no_build,
             &no_binary,
             concurrency,
+            preview,
         )
         .with_options(OptionsBuilder::new().exclude_newer(exclude_newer).build())
     };
@@ -415,6 +421,7 @@ pub(crate) async fn pip_install(
         &venv,
         dry_run,
         printer,
+        preview,
     )
     .await?;
 
