@@ -9,8 +9,7 @@ use owo_colors::OwoColorize;
 use tracing::debug;
 
 use distribution_types::{
-    CachedDist, Diagnostic, InstalledDist, Requirement, ResolutionDiagnostic,
-    UnresolvedRequirementSpecification,
+    CachedDist, Diagnostic, InstalledDist, ResolutionDiagnostic, UnresolvedRequirementSpecification,
 };
 use distribution_types::{
     DistributionMetadata, IndexLocations, InstalledMetadata, LocalDist, Name, Resolution,
@@ -18,10 +17,12 @@ use distribution_types::{
 use install_wheel_rs::linker::LinkMode;
 use pep508_rs::MarkerEnvironment;
 use platform_tags::Tags;
+use pypi_types::Requirement;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, RegistryClient};
 use uv_configuration::{
-    Concurrency, Constraints, NoBinary, Overrides, PreviewMode, Reinstall, Upgrade,
+    Concurrency, Constraints, ExtrasSpecification, NoBinary, Overrides, PreviewMode, Reinstall,
+    Upgrade,
 };
 use uv_dispatch::BuildDispatch;
 use uv_distribution::DistributionDatabase;
@@ -30,8 +31,8 @@ use uv_installer::{Downloader, Plan, Planner, SitePackages};
 use uv_interpreter::{Interpreter, PythonEnvironment};
 use uv_normalize::PackageName;
 use uv_requirements::{
-    ExtrasSpecification, LookaheadResolver, NamedRequirementsResolver, RequirementsSource,
-    RequirementsSpecification, SourceTreeResolver, Workspace,
+    LookaheadResolver, NamedRequirementsResolver, RequirementsSource, RequirementsSpecification,
+    SourceTreeResolver,
 };
 use uv_resolver::{
     DependencyMode, Exclusions, FlatIndex, InMemoryIndex, Manifest, Options, Preference,
@@ -41,8 +42,7 @@ use uv_types::{HashStrategy, InFlight, InstalledPackagesProvider};
 use uv_warnings::warn_user;
 
 use crate::commands::reporters::{DownloadReporter, InstallReporter, ResolverReporter};
-use crate::commands::DryRunEvent;
-use crate::commands::{compile_bytecode, elapsed, ChangeEvent, ChangeEventKind};
+use crate::commands::{compile_bytecode, elapsed, ChangeEvent, ChangeEventKind, DryRunEvent};
 use crate::printer::Printer;
 
 /// Consolidate the requirements for an installation.
@@ -50,10 +50,8 @@ pub(crate) async fn read_requirements(
     requirements: &[RequirementsSource],
     constraints: &[RequirementsSource],
     overrides: &[RequirementsSource],
-    workspace: Option<&Workspace>,
     extras: &ExtrasSpecification,
     client_builder: &BaseClientBuilder<'_>,
-    preview: PreviewMode,
 ) -> Result<RequirementsSpecification, Error> {
     // If the user requests `extras` but does not provide a valid source (e.g., a `pyproject.toml`),
     // return an error.
@@ -65,39 +63,13 @@ pub(crate) async fn read_requirements(
     }
 
     // Read all requirements from the provided sources.
-    let spec = RequirementsSpecification::from_sources(
+    Ok(RequirementsSpecification::from_sources(
         requirements,
         constraints,
         overrides,
-        workspace,
-        extras,
         client_builder,
-        preview,
     )
-    .await?;
-
-    // If all the metadata could be statically resolved, validate that every extra was used. If we
-    // need to resolve metadata via PEP 517, we don't know which extras are used until much later.
-    if spec.source_trees.is_empty() {
-        if let ExtrasSpecification::Some(extras) = extras {
-            let mut unused_extras = extras
-                .iter()
-                .filter(|extra| !spec.extras.contains(extra))
-                .collect::<Vec<_>>();
-            if !unused_extras.is_empty() {
-                unused_extras.sort_unstable();
-                unused_extras.dedup();
-                let s = if unused_extras.len() == 1 { "" } else { "s" };
-                return Err(anyhow!(
-                    "Requested extra{s} not found: {}",
-                    unused_extras.iter().join(", ")
-                )
-                .into());
-            }
-        }
-    }
-
-    Ok(spec)
+    .await?)
 }
 
 /// Resolve a set of requirements, similar to running `pip compile`.
@@ -107,15 +79,16 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
     constraints: Vec<Requirement>,
     overrides: Vec<UnresolvedRequirementSpecification>,
     source_trees: Vec<PathBuf>,
-    project: Option<PackageName>,
+    mut project: Option<PackageName>,
     extras: &ExtrasSpecification,
+    preferences: Vec<Preference>,
     installed_packages: InstalledPackages,
     hasher: &HashStrategy,
     reinstall: &Reinstall,
     upgrade: &Upgrade,
     interpreter: &Interpreter,
     tags: &Tags,
-    markers: &MarkerEnvironment,
+    markers: Option<&MarkerEnvironment>,
     client: &RegistryClient,
     flat_index: &FlatIndex,
     index: &InMemoryIndex,
@@ -123,6 +96,7 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
     concurrency: Concurrency,
     options: Options,
     printer: Printer,
+    preview: PreviewMode,
 ) -> Result<ResolutionGraph, Error> {
     let start = std::time::Instant::now();
 
@@ -133,7 +107,7 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
             requirements,
             hasher,
             index,
-            DistributionDatabase::new(client, build_dispatch, concurrency.downloads),
+            DistributionDatabase::new(client, build_dispatch, concurrency.downloads, preview),
         )
         .with_reporter(ResolverReporter::from(printer))
         .resolve()
@@ -141,17 +115,53 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
 
         // Resolve any source trees into requirements.
         if !source_trees.is_empty() {
+            let resolutions = SourceTreeResolver::new(
+                source_trees,
+                extras,
+                hasher,
+                index,
+                DistributionDatabase::new(client, build_dispatch, concurrency.downloads, preview),
+            )
+            .with_reporter(ResolverReporter::from(printer))
+            .resolve()
+            .await?;
+
+            // If we resolved a single project, use it for the project name.
+            project = project.or_else(|| {
+                if let [resolution] = &resolutions[..] {
+                    Some(resolution.project.clone())
+                } else {
+                    None
+                }
+            });
+
+            // If any of the extras were unused, surface a warning.
+            if let ExtrasSpecification::Some(extras) = extras {
+                let mut unused_extras = extras
+                    .iter()
+                    .filter(|extra| {
+                        !resolutions
+                            .iter()
+                            .any(|resolution| resolution.extras.contains(extra))
+                    })
+                    .collect::<Vec<_>>();
+                if !unused_extras.is_empty() {
+                    unused_extras.sort_unstable();
+                    unused_extras.dedup();
+                    let s = if unused_extras.len() == 1 { "" } else { "s" };
+                    return Err(anyhow!(
+                        "Requested extra{s} not found: {}",
+                        unused_extras.iter().join(", ")
+                    )
+                    .into());
+                }
+            }
+
+            // Extend the requirements with the resolved source trees.
             requirements.extend(
-                SourceTreeResolver::new(
-                    source_trees,
-                    extras,
-                    hasher,
-                    index,
-                    DistributionDatabase::new(client, build_dispatch, concurrency.downloads),
-                )
-                .with_reporter(ResolverReporter::from(printer))
-                .resolve()
-                .await?,
+                resolutions
+                    .into_iter()
+                    .flat_map(|resolution| resolution.requirements),
             );
         }
 
@@ -163,7 +173,7 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
         overrides,
         hasher,
         index,
-        DistributionDatabase::new(client, build_dispatch, concurrency.downloads),
+        DistributionDatabase::new(client, build_dispatch, concurrency.downloads, preview),
     )
     .with_reporter(ResolverReporter::from(printer))
     .resolve()
@@ -172,7 +182,11 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
     // Collect constraints and overrides.
     let constraints = Constraints::from_requirements(constraints);
     let overrides = Overrides::from_requirements(overrides);
-    let python_requirement = PythonRequirement::from_marker_environment(interpreter, markers);
+    let python_requirement = if let Some(markers) = markers {
+        PythonRequirement::from_marker_environment(interpreter, markers)
+    } else {
+        PythonRequirement::from_interpreter(interpreter)
+    };
 
     // Determine any lookahead requirements.
     let lookaheads = match options.dependency_mode {
@@ -183,10 +197,10 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
                 &overrides,
                 hasher,
                 index,
-                DistributionDatabase::new(client, build_dispatch, concurrency.downloads),
+                DistributionDatabase::new(client, build_dispatch, concurrency.downloads, preview),
             )
             .with_reporter(ResolverReporter::from(printer))
-            .resolve(Some(markers))
+            .resolve(markers)
             .await?
         }
         DependencyMode::Direct => Vec::new(),
@@ -195,13 +209,6 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
     // TODO(zanieb): Consider consuming these instead of cloning
     let exclusions = Exclusions::new(reinstall.clone(), upgrade.clone());
 
-    // Prefer current site packages; filter out packages that are marked for reinstall or upgrade.
-    let preferences = installed_packages
-        .iter()
-        .filter(|dist| !exclusions.contains(dist.name()))
-        .map(Preference::from_installed)
-        .collect::<Vec<_>>();
-
     // Create a manifest of the requirements.
     let manifest = Manifest::new(
         requirements,
@@ -227,14 +234,14 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
             manifest,
             options,
             &python_requirement,
-            Some(markers),
+            markers,
             tags,
             flat_index,
             index,
             hasher,
             build_dispatch,
             installed_packages,
-            DistributionDatabase::new(client, build_dispatch, concurrency.downloads),
+            DistributionDatabase::new(client, build_dispatch, concurrency.downloads, preview),
         )?
         .with_reporter(reporter);
 
@@ -292,6 +299,7 @@ pub(crate) async fn install(
     venv: &PythonEnvironment,
     dry_run: bool,
     printer: Printer,
+    preview: PreviewMode,
 ) -> Result<(), Error> {
     let start = std::time::Instant::now();
 
@@ -367,7 +375,7 @@ pub(crate) async fn install(
             cache,
             tags,
             hasher,
-            DistributionDatabase::new(client, build_dispatch, concurrency.downloads),
+            DistributionDatabase::new(client, build_dispatch, concurrency.downloads, preview),
         )
         .with_reporter(DownloadReporter::from(printer).with_length(remote.len() as u64));
 
@@ -721,6 +729,9 @@ pub(crate) enum Error {
     #[error(transparent)]
     Lookahead(#[from] uv_requirements::LookaheadError),
 
+    #[error(transparent)]
+    Named(#[from] uv_requirements::NamedRequirementsError),
+
     #[error(transparent)]
     Anyhow(#[from] anyhow::Error),
 }
