@@ -13,15 +13,16 @@ use uv_auth::store_credentials_from_url;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{
-    Concurrency, ConfigSettings, IndexStrategy, NoBinary, NoBuild, PreviewMode, Reinstall,
-    SetupPyStrategy, Upgrade,
+    Concurrency, ConfigSettings, ExtrasSpecification, IndexStrategy, NoBinary, NoBuild,
+    PreviewMode, Reinstall, SetupPyStrategy, Upgrade,
 };
 use uv_configuration::{KeyringProviderType, TargetTriple};
 use uv_dispatch::BuildDispatch;
 use uv_fs::Simplified;
+use uv_git::GitResolver;
 use uv_installer::SitePackages;
 use uv_interpreter::{PythonEnvironment, PythonVersion, SystemPython, Target};
-use uv_requirements::{ExtrasSpecification, RequirementsSource, RequirementsSpecification};
+use uv_requirements::{RequirementsSource, RequirementsSpecification};
 use uv_resolver::{
     DependencyMode, ExcludeNewer, FlatIndex, InMemoryIndex, OptionsBuilder, PreReleaseMode,
     ResolutionMode,
@@ -97,10 +98,8 @@ pub(crate) async fn pip_sync(
         requirements,
         constraints,
         overrides,
-        None,
-        &ExtrasSpecification::default(),
+        &extras,
         &client_builder,
-        preview,
     )
     .await?;
 
@@ -253,6 +252,10 @@ pub(crate) async fn pip_sync(
     // Track in-flight downloads, builds, etc., across resolutions.
     let in_flight = InFlight::default();
 
+    // When resolving, don't take any external preferences into account.
+    let preferences = Vec::default();
+    let git = GitResolver::default();
+
     // Create a build dispatch for resolution.
     let resolve_dispatch = BuildDispatch::new(
         &client,
@@ -261,6 +264,7 @@ pub(crate) async fn pip_sync(
         &index_locations,
         &flat_index,
         &index,
+        &git,
         &in_flight,
         setup_py,
         config_settings,
@@ -269,6 +273,7 @@ pub(crate) async fn pip_sync(
         &no_build,
         &no_binary,
         concurrency,
+        preview,
     )
     .with_options(OptionsBuilder::new().exclude_newer(exclude_newer).build());
 
@@ -290,13 +295,14 @@ pub(crate) async fn pip_sync(
         source_trees,
         project,
         &extras,
+        preferences,
         site_packages.clone(),
         &hasher,
         reinstall,
         &upgrade,
         interpreter,
         &tags,
-        &markers,
+        Some(&markers),
         &client,
         &flat_index,
         &index,
@@ -304,6 +310,7 @@ pub(crate) async fn pip_sync(
         concurrency,
         options,
         printer,
+        preview,
     )
     .await
     {
@@ -332,6 +339,7 @@ pub(crate) async fn pip_sync(
             &index_locations,
             &flat_index,
             &index,
+            &git,
             &in_flight,
             setup_py,
             config_settings,
@@ -340,6 +348,7 @@ pub(crate) async fn pip_sync(
             &no_build,
             &no_binary,
             concurrency,
+            preview,
         )
         .with_options(OptionsBuilder::new().exclude_newer(exclude_newer).build())
     };
@@ -364,6 +373,7 @@ pub(crate) async fn pip_sync(
         &venv,
         dry_run,
         printer,
+        preview,
     )
     .await?;
 
