@@ -5,25 +5,25 @@ use itertools::Itertools;
 use owo_colors::OwoColorize;
 use tracing::debug;
 
-use crate::commands::pip;
 use distribution_types::{IndexLocations, Resolution};
 use install_wheel_rs::linker::LinkMode;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity, RegistryClientBuilder};
 use uv_configuration::{
-    Concurrency, ConfigSettings, NoBinary, NoBuild, PreviewMode, Reinstall, SetupPyStrategy,
-    Upgrade,
+    Concurrency, ConfigSettings, ExtrasSpecification, NoBinary, NoBuild, PreviewMode, Reinstall,
+    SetupPyStrategy, Upgrade,
 };
 use uv_dispatch::BuildDispatch;
+use uv_distribution::ProjectWorkspace;
 use uv_fs::Simplified;
+use uv_git::GitResolver;
 use uv_installer::{SatisfiesResult, SitePackages};
 use uv_interpreter::{find_default_interpreter, PythonEnvironment};
-use uv_requirements::{
-    ExtrasSpecification, ProjectWorkspace, RequirementsSource, RequirementsSpecification, Workspace,
-};
+use uv_requirements::{RequirementsSource, RequirementsSpecification};
 use uv_resolver::{FlatIndex, InMemoryIndex, Options};
 use uv_types::{BuildIsolation, HashStrategy, InFlight};
 
+use crate::commands::pip;
 use crate::printer::Printer;
 
 pub(crate) mod lock;
@@ -31,15 +31,33 @@ pub(crate) mod run;
 pub(crate) mod sync;
 
 #[derive(thiserror::Error, Debug)]
-pub(crate) enum Error {
+pub(crate) enum ProjectError {
     #[error(transparent)]
     Interpreter(#[from] uv_interpreter::Error),
 
     #[error(transparent)]
     Virtualenv(#[from] uv_virtualenv::Error),
 
+    #[error(transparent)]
+    Tags(#[from] platform_tags::TagsError),
+
+    #[error(transparent)]
+    Lock(#[from] uv_resolver::LockError),
+
     #[error(transparent)]
     Fmt(#[from] std::fmt::Error),
+
+    #[error(transparent)]
+    Io(#[from] std::io::Error),
+
+    #[error(transparent)]
+    Serialize(#[from] toml::ser::Error),
+
+    #[error(transparent)]
+    Anyhow(#[from] anyhow::Error),
+
+    #[error(transparent)]
+    Operation(#[from] pip::operations::Error),
 }
 
 /// Initialize a virtual environment for the current project.
@@ -48,7 +66,7 @@ pub(crate) fn init_environment(
     preview: PreviewMode,
     cache: &Cache,
     printer: Printer,
-) -> Result<PythonEnvironment, Error> {
+) -> Result<PythonEnvironment, ProjectError> {
     let venv = project.workspace().root().join(".venv");
 
     // Discover or create the virtual environment.
@@ -91,11 +109,11 @@ pub(crate) fn init_environment(
 pub(crate) async fn update_environment(
     venv: PythonEnvironment,
     requirements: &[RequirementsSource],
-    workspace: Option<&Workspace>,
-    preview: PreviewMode,
+    index_locations: &IndexLocations,
     connectivity: Connectivity,
     cache: &Cache,
     printer: Printer,
+    preview: PreviewMode,
 ) -> Result<PythonEnvironment> {
     // TODO(zanieb): Support client configuration
     let client_builder = BaseClientBuilder::default().connectivity(connectivity);
@@ -103,16 +121,8 @@ pub(crate) async fn update_environment(
     // Read all requirements from the provided sources.
     // TODO(zanieb): Consider allowing constraints and extras
     // TODO(zanieb): Allow specifying extras somehow
-    let spec = RequirementsSpecification::from_sources(
-        requirements,
-        &[],
-        &[],
-        workspace,
-        &ExtrasSpecification::None,
-        &client_builder,
-        preview,
-    )
-    .await?;
+    let spec =
+        RequirementsSpecification::from_sources(requirements, &[], &[], &client_builder).await?;
 
     // Check if the current environment satisfies the requirements
     let site_packages = SitePackages::from_executable(&venv)?;
@@ -147,38 +157,41 @@ pub(crate) async fn update_environment(
     // TODO(zanieb): Support client options e.g. offline, tls, etc.
     let client = RegistryClientBuilder::new(cache.clone())
         .connectivity(connectivity)
+        .index_urls(index_locations.index_urls())
         .markers(markers)
         .platform(venv.interpreter().platform())
         .build();
 
     // TODO(charlie): Respect project configuration.
     let build_isolation = BuildIsolation::default();
+    let compile = false;
+    let concurrency = Concurrency::default();
     let config_settings = ConfigSettings::default();
+    let dry_run = false;
+    let extras = ExtrasSpecification::default();
     let flat_index = FlatIndex::default();
+    let git = GitResolver::default();
     let hasher = HashStrategy::default();
     let in_flight = InFlight::default();
     let index = InMemoryIndex::default();
-    let index_locations = IndexLocations::default();
     let link_mode = LinkMode::default();
     let no_binary = NoBinary::default();
     let no_build = NoBuild::default();
-    let setup_py = SetupPyStrategy::default();
-    let concurrency = Concurrency::default();
-    let reinstall = Reinstall::default();
-    let compile = false;
-    let dry_run = false;
-    let extras = ExtrasSpecification::default();
-    let upgrade = Upgrade::default();
     let options = Options::default();
+    let preferences = Vec::default();
+    let reinstall = Reinstall::default();
+    let setup_py = SetupPyStrategy::default();
+    let upgrade = Upgrade::default();
 
     // Create a build dispatch.
     let resolve_dispatch = BuildDispatch::new(
         &client,
         cache,
         &interpreter,
-        &index_locations,
+        index_locations,
         &flat_index,
         &index,
+        &git,
         &in_flight,
         setup_py,
         &config_settings,
@@ -187,6 +200,7 @@ pub(crate) async fn update_environment(
         &no_build,
         &no_binary,
         concurrency,
+        preview,
     );
 
     // Resolve the requirements.
@@ -197,13 +211,14 @@ pub(crate) async fn update_environment(
         spec.source_trees,
         spec.project,
         &extras,
+        preferences,
         site_packages.clone(),
         &hasher,
         &reinstall,
         &upgrade,
         &interpreter,
         tags,
-        markers,
+        Some(markers),
         &client,
         &flat_index,
         &index,
@@ -211,6 +226,7 @@ pub(crate) async fn update_environment(
         concurrency,
         options,
         printer,
+        preview,
     )
     .await
     {
@@ -230,9 +246,10 @@ pub(crate) async fn update_environment(
             &client,
             cache,
             &interpreter,
-            &index_locations,
+            index_locations,
             &flat_index,
             &index,
+            &git,
             &in_flight,
             setup_py,
             &config_settings,
@@ -241,6 +258,7 @@ pub(crate) async fn update_environment(
             &no_build,
             &no_binary,
             concurrency,
+            preview,
         )
     };
 
@@ -253,7 +271,7 @@ pub(crate) async fn update_environment(
         &no_binary,
         link_mode,
         compile,
-        &index_locations,
+        index_locations,
         &hasher,
         tags,
         &client,
@@ -264,6 +282,7 @@ pub(crate) async fn update_environment(
         &venv,
         dry_run,
         printer,
+        preview,
     )
     .await?;
 
