@@ -2,6 +2,7 @@ use std::ffi::OsString;
 use std::path::PathBuf;
 
 use anyhow::{Context, Result};
+use distribution_types::IndexLocations;
 use itertools::Itertools;
 use tempfile::tempdir_in;
 use tokio::process::Command;
@@ -9,9 +10,12 @@ use tracing::debug;
 
 use uv_cache::Cache;
 use uv_client::Connectivity;
-use uv_configuration::PreviewMode;
+use uv_configuration::{ExtrasSpecification, PreviewMode, Upgrade};
+use uv_distribution::{ProjectWorkspace, Workspace};
 use uv_interpreter::{PythonEnvironment, SystemPython};
-use uv_requirements::{ProjectWorkspace, RequirementsSource};
+use uv_normalize::PackageName;
+use uv_requirements::RequirementsSource;
+use uv_resolver::ExcludeNewer;
 use uv_warnings::warn_user;
 
 use crate::commands::{project, ExitStatus};
@@ -20,10 +24,15 @@ use crate::printer::Printer;
 /// Run a command.
 #[allow(clippy::too_many_arguments)]
 pub(crate) async fn run(
+    index_locations: IndexLocations,
+    extras: ExtrasSpecification,
     target: Option<String>,
     mut args: Vec<OsString>,
     requirements: Vec<RequirementsSource>,
     python: Option<String>,
+    upgrade: Upgrade,
+    exclude_newer: Option<ExcludeNewer>,
+    package: Option<PackageName>,
     isolated: bool,
     preview: PreviewMode,
     connectivity: Connectivity,
@@ -34,45 +43,50 @@ pub(crate) async fn run(
         warn_user!("`uv run` is experimental and may change without warning.");
     }
 
-    let command = if let Some(target) = target {
-        let target_path = PathBuf::from(&target);
-        if target_path
-            .extension()
-            .map_or(false, |ext| ext.eq_ignore_ascii_case("py"))
-            && target_path.exists()
-        {
-            args.insert(0, target_path.as_os_str().into());
-            "python".to_string()
-        } else {
-            target
-        }
-    } else {
-        "python".to_string()
-    };
-
     // Discover and sync the project.
     let project_env = if isolated {
+        // package is `None`, isolated and package are marked as conflicting in clap.
         None
     } else {
         debug!("Syncing project environment.");
 
-        let project = ProjectWorkspace::discover(std::env::current_dir()?).await?;
-
+        let project = if let Some(package) = package {
+            // We need a workspace, but we don't need to have a current package, we can be e.g. in
+            // the root of a virtual workspace and then switch into the selected package.
+            Workspace::discover(&std::env::current_dir()?, None)
+                .await?
+                .with_current_project(package.clone())
+                .with_context(|| format!("Package `{package}` not found in workspace"))?
+        } else {
+            ProjectWorkspace::discover(&std::env::current_dir()?, None).await?
+        };
         let venv = project::init_environment(&project, preview, cache, printer)?;
 
-        // Install the project requirements.
-        Some(
-            project::update_environment(
-                venv,
-                &project.requirements(),
-                Some(project.workspace()),
-                preview,
-                connectivity,
-                cache,
-                printer,
-            )
-            .await?,
+        // Lock and sync the environment.
+        let lock = project::lock::do_lock(
+            &project,
+            &venv,
+            &index_locations,
+            upgrade,
+            exclude_newer,
+            preview,
+            cache,
+            printer,
         )
+        .await?;
+        project::sync::do_sync(
+            &project,
+            &venv,
+            &lock,
+            &index_locations,
+            extras,
+            preview,
+            cache,
+            printer,
+        )
+        .await?;
+
+        Some(venv)
     };
 
     // If necessary, create an environment for the ephemeral requirements.
@@ -115,17 +129,33 @@ pub(crate) async fn run(
             project::update_environment(
                 venv,
                 &requirements,
-                None,
-                preview,
+                &index_locations,
                 connectivity,
                 cache,
                 printer,
+                preview,
             )
             .await?,
         )
     };
 
     // Construct the command
+    let command = if let Some(target) = target {
+        let target_path = PathBuf::from(&target);
+        if target_path
+            .extension()
+            .map_or(false, |ext| ext.eq_ignore_ascii_case("py"))
+            && target_path.exists()
+        {
+            args.insert(0, target_path.as_os_str().into());
+            "python".to_string()
+        } else {
+            target
+        }
+    } else {
+        "python".to_string()
+    };
+
     let mut process = Command::new(&command);
     process.args(&args);
 
