@@ -205,6 +205,7 @@ async fn run() -> Result<ExitStatus> {
                 &requirements,
                 &constraints,
                 &overrides,
+                args.overrides_from_workspace,
                 args.shared.extras,
                 args.shared.output_file.as_deref(),
                 args.shared.resolution,
@@ -237,7 +238,6 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.python,
                 args.shared.system,
                 args.shared.concurrency,
-                args.uv_lock,
                 globals.native_tls,
                 globals.quiet,
                 globals.preview,
@@ -345,6 +345,7 @@ async fn run() -> Result<ExitStatus> {
                 &requirements,
                 &constraints,
                 &overrides,
+                args.overrides_from_workspace,
                 &args.shared.extras,
                 args.shared.resolution,
                 args.shared.prerelease,
@@ -372,7 +373,6 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.break_system_packages,
                 args.shared.target,
                 args.shared.concurrency,
-                args.uv_lock,
                 globals.native_tls,
                 globals.preview,
                 cache,
@@ -550,7 +550,7 @@ async fn run() -> Result<ExitStatus> {
             let args = settings::RunSettings::resolve(args, workspace);
 
             // Initialize the cache.
-            let cache = cache.init()?;
+            let cache = cache.init()?.with_refresh(args.refresh);
 
             let requirements = args
                 .with
@@ -573,10 +573,15 @@ async fn run() -> Result<ExitStatus> {
                 .collect::<Vec<_>>();
 
             commands::run(
+                args.index_locations,
+                args.extras,
                 args.target,
                 args.args,
                 requirements,
                 args.python,
+                args.upgrade,
+                args.exclude_newer,
+                args.package,
                 globals.isolated,
                 globals.preview,
                 globals.connectivity,
@@ -587,21 +592,36 @@ async fn run() -> Result<ExitStatus> {
         }
         Commands::Sync(args) => {
             // Resolve the settings from the command-line arguments and workspace configuration.
-            let _args = settings::SyncSettings::resolve(args, workspace);
+            let args = settings::SyncSettings::resolve(args, workspace);
 
             // Initialize the cache.
-            let cache = cache.init()?;
+            let cache = cache.init()?.with_refresh(args.refresh);
 
-            commands::sync(globals.preview, &cache, printer).await
+            commands::sync(
+                args.index_locations,
+                args.extras,
+                globals.preview,
+                &cache,
+                printer,
+            )
+            .await
         }
         Commands::Lock(args) => {
             // Resolve the settings from the command-line arguments and workspace configuration.
-            let _args = settings::LockSettings::resolve(args, workspace);
+            let args = settings::LockSettings::resolve(args, workspace);
 
             // Initialize the cache.
-            let cache = cache.init()?;
+            let cache = cache.init()?.with_refresh(args.refresh);
 
-            commands::lock(globals.preview, &cache, printer).await
+            commands::lock(
+                args.index_locations,
+                args.upgrade,
+                args.exclude_newer,
+                globals.preview,
+                &cache,
+                printer,
+            )
+            .await
         }
         #[cfg(feature = "self-update")]
         Commands::Self_(SelfNamespace {
@@ -618,6 +638,9 @@ async fn run() -> Result<ExitStatus> {
         Commands::Tool(ToolNamespace {
             command: ToolCommand::Run(args),
         }) => {
+            // Resolve the settings from the command-line arguments and workspace configuration.
+            let args = settings::ToolRunSettings::resolve(args, workspace);
+
             // Initialize the cache.
             let cache = cache.init()?;
 
@@ -629,6 +652,7 @@ async fn run() -> Result<ExitStatus> {
                 args.with,
                 globals.isolated,
                 globals.preview,
+                args.index_locations,
                 globals.connectivity,
                 &cache,
                 printer,
