@@ -7,22 +7,23 @@ use std::str::FromStr;
 
 use distribution_types::IndexLocations;
 use install_wheel_rs::linker::LinkMode;
+use pep508_rs::RequirementOrigin;
+use pypi_types::Requirement;
 use uv_cache::{CacheArgs, Refresh};
 use uv_client::Connectivity;
 use uv_configuration::{
-    Concurrency, ConfigSettings, IndexStrategy, KeyringProviderType, NoBinary, NoBuild,
-    PreviewMode, Reinstall, SetupPyStrategy, TargetTriple, Upgrade,
+    Concurrency, ConfigSettings, ExtrasSpecification, IndexStrategy, KeyringProviderType, NoBinary,
+    NoBuild, PreviewMode, Reinstall, SetupPyStrategy, TargetTriple, Upgrade,
 };
 use uv_interpreter::{PythonVersion, Target};
 use uv_normalize::PackageName;
-use uv_requirements::ExtrasSpecification;
 use uv_resolver::{AnnotationStyle, DependencyMode, ExcludeNewer, PreReleaseMode, ResolutionMode};
 use uv_workspace::{Combine, PipOptions, Workspace};
 
 use crate::cli::{
     ColorChoice, GlobalArgs, LockArgs, Maybe, PipCheckArgs, PipCompileArgs, PipFreezeArgs,
     PipInstallArgs, PipListArgs, PipShowArgs, PipSyncArgs, PipUninstallArgs, RunArgs, SyncArgs,
-    VenvArgs,
+    ToolRunArgs, VenvArgs,
 };
 use crate::commands::ListFormat;
 
@@ -98,11 +99,16 @@ impl CacheSettings {
 #[allow(clippy::struct_excessive_bools)]
 #[derive(Debug, Clone)]
 pub(crate) struct RunSettings {
-    // CLI-only settings.
+    pub(crate) index_locations: IndexLocations,
+    pub(crate) extras: ExtrasSpecification,
     pub(crate) target: Option<String>,
     pub(crate) args: Vec<OsString>,
     pub(crate) with: Vec<String>,
     pub(crate) python: Option<String>,
+    pub(crate) refresh: Refresh,
+    pub(crate) upgrade: Upgrade,
+    pub(crate) exclude_newer: Option<ExcludeNewer>,
+    pub(crate) package: Option<PackageName>,
 }
 
 impl RunSettings {
@@ -110,16 +116,99 @@ impl RunSettings {
     #[allow(clippy::needless_pass_by_value)]
     pub(crate) fn resolve(args: RunArgs, _workspace: Option<Workspace>) -> Self {
         let RunArgs {
+            extra,
+            all_extras,
+            no_all_extras,
+            target,
+            args,
+            with,
+            refresh,
+            no_refresh,
+            refresh_package,
+            upgrade,
+            no_upgrade,
+            upgrade_package,
+
+            index_args,
+            python,
+            exclude_newer,
+            package,
+        } = args;
+
+        Self {
+            index_locations: IndexLocations::new(
+                index_args.index_url.and_then(Maybe::into_option),
+                index_args
+                    .extra_index_url
+                    .map(|extra_index_urls| {
+                        extra_index_urls
+                            .into_iter()
+                            .filter_map(Maybe::into_option)
+                            .collect()
+                    })
+                    .unwrap_or_default(),
+                index_args.find_links.unwrap_or_default(),
+                index_args.no_index,
+            ),
+            refresh: Refresh::from_args(flag(refresh, no_refresh), refresh_package),
+            upgrade: Upgrade::from_args(flag(upgrade, no_upgrade), upgrade_package),
+            extras: ExtrasSpecification::from_args(
+                flag(all_extras, no_all_extras).unwrap_or_default(),
+                extra.unwrap_or_default(),
+            ),
             target,
             args,
             with,
             python,
+            exclude_newer,
+            package,
+        }
+    }
+}
+
+/// The resolved settings to use for a `tool run` invocation.
+#[allow(clippy::struct_excessive_bools)]
+#[derive(Debug, Clone)]
+pub(crate) struct ToolRunSettings {
+    pub(crate) index_locations: IndexLocations,
+    pub(crate) target: String,
+    pub(crate) args: Vec<OsString>,
+    pub(crate) from: Option<String>,
+    pub(crate) with: Vec<String>,
+    pub(crate) python: Option<String>,
+}
+
+impl ToolRunSettings {
+    /// Resolve the [`ToolRunSettings`] from the CLI and workspace configuration.
+    #[allow(clippy::needless_pass_by_value)]
+    pub(crate) fn resolve(args: ToolRunArgs, _workspace: Option<Workspace>) -> Self {
+        let ToolRunArgs {
+            target,
+            args,
+            from,
+            with,
+            index_args,
+            python,
         } = args;
 
         Self {
-            // CLI-only settings.
+            index_locations: IndexLocations::new(
+                index_args.index_url.and_then(Maybe::into_option),
+                index_args
+                    .extra_index_url
+                    .map(|extra_index_urls| {
+                        extra_index_urls
+                            .into_iter()
+                            .filter_map(Maybe::into_option)
+                            .collect()
+                    })
+                    .unwrap_or_default(),
+                index_args.find_links.unwrap_or_default(),
+                index_args.no_index,
+            ),
             target,
             args,
+            from,
             with,
             python,
         }
@@ -130,7 +219,9 @@ impl RunSettings {
 #[allow(clippy::struct_excessive_bools, dead_code)]
 #[derive(Debug, Clone)]
 pub(crate) struct SyncSettings {
-    // CLI-only settings.
+    pub(crate) index_locations: IndexLocations,
+    pub(crate) refresh: Refresh,
+    pub(crate) extras: ExtrasSpecification,
     pub(crate) python: Option<String>,
 }
 
@@ -138,10 +229,37 @@ impl SyncSettings {
     /// Resolve the [`SyncSettings`] from the CLI and workspace configuration.
     #[allow(clippy::needless_pass_by_value)]
     pub(crate) fn resolve(args: SyncArgs, _workspace: Option<Workspace>) -> Self {
-        let SyncArgs { python } = args;
+        let SyncArgs {
+            extra,
+            all_extras,
+            no_all_extras,
+            refresh,
+            no_refresh,
+            refresh_package,
+            index_args,
+            python,
+        } = args;
 
         Self {
-            // CLI-only settings.
+            index_locations: IndexLocations::new(
+                index_args.index_url.and_then(Maybe::into_option),
+                index_args
+                    .extra_index_url
+                    .map(|extra_index_urls| {
+                        extra_index_urls
+                            .into_iter()
+                            .filter_map(Maybe::into_option)
+                            .collect()
+                    })
+                    .unwrap_or_default(),
+                index_args.find_links.unwrap_or_default(),
+                index_args.no_index,
+            ),
+            refresh: Refresh::from_args(flag(refresh, no_refresh), refresh_package),
+            extras: ExtrasSpecification::from_args(
+                flag(all_extras, no_all_extras).unwrap_or_default(),
+                extra.unwrap_or_default(),
+            ),
             python,
         }
     }
@@ -151,7 +269,10 @@ impl SyncSettings {
 #[allow(clippy::struct_excessive_bools, dead_code)]
 #[derive(Debug, Clone)]
 pub(crate) struct LockSettings {
-    // CLI-only settings.
+    pub(crate) index_locations: IndexLocations,
+    pub(crate) refresh: Refresh,
+    pub(crate) upgrade: Upgrade,
+    pub(crate) exclude_newer: Option<ExcludeNewer>,
     pub(crate) python: Option<String>,
 }
 
@@ -159,10 +280,36 @@ impl LockSettings {
     /// Resolve the [`LockSettings`] from the CLI and workspace configuration.
     #[allow(clippy::needless_pass_by_value)]
     pub(crate) fn resolve(args: LockArgs, _workspace: Option<Workspace>) -> Self {
-        let LockArgs { python } = args;
+        let LockArgs {
+            refresh,
+            no_refresh,
+            refresh_package,
+            upgrade,
+            no_upgrade,
+            upgrade_package,
+            index_args,
+            exclude_newer,
+            python,
+        } = args;
 
         Self {
-            // CLI-only settings.
+            index_locations: IndexLocations::new(
+                index_args.index_url.and_then(Maybe::into_option),
+                index_args
+                    .extra_index_url
+                    .map(|extra_index_urls| {
+                        extra_index_urls
+                            .into_iter()
+                            .filter_map(Maybe::into_option)
+                            .collect()
+                    })
+                    .unwrap_or_default(),
+                index_args.find_links.unwrap_or_default(),
+                index_args.no_index,
+            ),
+            refresh: Refresh::from_args(flag(refresh, no_refresh), refresh_package),
+            upgrade: Upgrade::from_args(flag(upgrade, no_upgrade), upgrade_package),
+            exclude_newer,
             python,
         }
     }
@@ -178,10 +325,11 @@ pub(crate) struct PipCompileSettings {
     pub(crate) r#override: Vec<PathBuf>,
     pub(crate) refresh: Refresh,
     pub(crate) upgrade: Upgrade,
-    pub(crate) uv_lock: bool,
 
     // Shared settings.
     pub(crate) shared: PipSharedSettings,
+    // Override dependencies from workspace.
+    pub(crate) overrides_from_workspace: Vec<Requirement>,
 }
 
 impl PipCompileSettings {
@@ -208,17 +356,13 @@ impl PipCompileSettings {
             header,
             annotation_style,
             custom_compile_command,
-
             refresh,
             no_refresh,
             refresh_package,
             link_mode,
-            index_url,
-            extra_index_url,
-            no_index,
+            index_args,
             index_strategy,
             keyring_provider,
-            find_links,
             python,
             system,
             no_system,
@@ -247,11 +391,24 @@ impl PipCompileSettings {
             no_emit_marker_expression,
             emit_index_annotation,
             no_emit_index_annotation,
-            unstable_uv_lock_file,
-            no_unstable_uv_lock_file,
             compat_args: _,
         } = args;
 
+        let overrides_from_workspace: Vec<Requirement> = if let Some(workspace) = &workspace {
+            workspace
+                .options
+                .override_dependencies
+                .clone()
+                .unwrap_or_default()
+                .into_iter()
+                .map(|requirement| {
+                    Requirement::from(requirement.with_origin(RequirementOrigin::Workspace))
+                })
+                .collect()
+        } else {
+            Vec::new()
+        };
+
         Self {
             // CLI-only settings.
             src_file,
@@ -262,7 +419,7 @@ impl PipCompileSettings {
             r#override,
             refresh: Refresh::from_args(flag(refresh, no_refresh), refresh_package),
             upgrade: Upgrade::from_args(flag(upgrade, no_upgrade), upgrade_package),
-            uv_lock: flag(unstable_uv_lock_file, no_unstable_uv_lock_file).unwrap_or(false),
+            overrides_from_workspace,
 
             // Shared settings.
             shared: PipSharedSettings::combine(
@@ -270,15 +427,15 @@ impl PipCompileSettings {
                     python,
                     system: flag(system, no_system),
 
-                    index_url: index_url.and_then(Maybe::into_option),
-                    extra_index_url: extra_index_url.map(|extra_index_urls| {
+                    index_url: index_args.index_url.and_then(Maybe::into_option),
+                    extra_index_url: index_args.extra_index_url.map(|extra_index_urls| {
                         extra_index_urls
                             .into_iter()
                             .filter_map(Maybe::into_option)
                             .collect()
                     }),
-                    no_index: Some(no_index),
-                    find_links,
+                    no_index: Some(index_args.no_index),
+                    find_links: index_args.find_links,
                     index_strategy,
                     keyring_provider,
                     no_build: flag(no_build, build),
@@ -352,10 +509,7 @@ impl PipSyncSettings {
             no_refresh,
             refresh_package,
             link_mode,
-            index_url,
-            extra_index_url,
-            find_links,
-            no_index,
+            index_args,
             index_strategy,
             require_hashes,
             no_require_hashes,
@@ -404,16 +558,15 @@ impl PipSyncSettings {
                     system: flag(system, no_system),
                     break_system_packages: flag(break_system_packages, no_break_system_packages),
                     target,
-
-                    index_url: index_url.and_then(Maybe::into_option),
-                    extra_index_url: extra_index_url.map(|extra_index_urls| {
+                    index_url: index_args.index_url.and_then(Maybe::into_option),
+                    extra_index_url: index_args.extra_index_url.map(|extra_index_urls| {
                         extra_index_urls
                             .into_iter()
                             .filter_map(Maybe::into_option)
                             .collect()
                     }),
-                    no_index: Some(no_index),
-                    find_links,
+                    no_index: Some(index_args.no_index),
+                    find_links: index_args.find_links,
                     index_strategy,
                     keyring_provider,
                     no_build: flag(no_build, build),
@@ -456,7 +609,7 @@ pub(crate) struct PipInstallSettings {
     pub(crate) reinstall: Reinstall,
     pub(crate) refresh: Refresh,
     pub(crate) dry_run: bool,
-    pub(crate) uv_lock: Option<String>,
+    pub(crate) overrides_from_workspace: Vec<Requirement>,
 
     // Shared settings.
     pub(crate) shared: PipSharedSettings,
@@ -489,10 +642,7 @@ impl PipInstallSettings {
             resolution,
             prerelease,
             pre,
-            index_url,
-            extra_index_url,
-            find_links,
-            no_index,
+            index_args,
             index_strategy,
             require_hashes,
             no_require_hashes,
@@ -520,10 +670,24 @@ impl PipInstallSettings {
             no_strict,
             exclude_newer,
             dry_run,
-            unstable_uv_lock_file,
             compat_args: _,
         } = args;
 
+        let overrides_from_workspace: Vec<Requirement> = if let Some(workspace) = &workspace {
+            workspace
+                .options
+                .override_dependencies
+                .clone()
+                .unwrap_or_default()
+                .into_iter()
+                .map(|requirement| {
+                    Requirement::from(requirement.with_origin(RequirementOrigin::Workspace))
+                })
+                .collect()
+        } else {
+            Vec::new()
+        };
+
         Self {
             // CLI-only settings.
             package,
@@ -538,7 +702,7 @@ impl PipInstallSettings {
             reinstall: Reinstall::from_args(flag(reinstall, no_reinstall), reinstall_package),
             refresh: Refresh::from_args(flag(refresh, no_refresh), refresh_package),
             dry_run,
-            uv_lock: unstable_uv_lock_file,
+            overrides_from_workspace,
 
             // Shared settings.
             shared: PipSharedSettings::combine(
@@ -547,16 +711,15 @@ impl PipInstallSettings {
                     system: flag(system, no_system),
                     break_system_packages: flag(break_system_packages, no_break_system_packages),
                     target,
-
-                    index_url: index_url.and_then(Maybe::into_option),
-                    extra_index_url: extra_index_url.map(|extra_index_urls| {
+                    index_url: index_args.index_url.and_then(Maybe::into_option),
+                    extra_index_url: index_args.extra_index_url.map(|extra_index_urls| {
                         extra_index_urls
                             .into_iter()
                             .filter_map(Maybe::into_option)
                             .collect()
                     }),
-                    no_index: Some(no_index),
-                    find_links,
+                    no_index: Some(index_args.no_index),
+                    find_links: index_args.find_links,
                     index_strategy,
                     keyring_provider,
                     no_build: flag(no_build, build),
